import os.path
import matplotlib.pyplot as plt
import numpy as np
from matplotlib.ticker import MaxNLocator

#method to create a plot of the comparisons with mean and 95% confidence intervals
def createPlot(dataDict, filename = 'plot.pdf', figwidth=6, xLabel = '', showLineAtZero = True, showOnlyIntegers = False, alsoShowPlot = False, ratioData = False):
    #selection of plot markers to use when showing an interaction
    plotMarker = ['o', 's', 'D', 'v', '^', 'p', 'h', '*', 'x', '+']
    
    #calculate height of the plot
    #calculate if and how many values are shown for one series (if it is an interaction)
    seriesNamesFromDataDict = list(dataDict.keys())
    numberOfInteractionLevels = len(dataDict[seriesNamesFromDataDict[0]])
    figheight = 0.3 * len(dataDict) * numberOfInteractionLevels + 1
    
    #get the names of the factor levels
    listOfFactorLevelNames = []
    if numberOfInteractionLevels > 1:
        for i in range(numberOfInteractionLevels):
            if 'factorLevel' in dataDict[seriesNamesFromDataDict[0]][i]:
                listOfFactorLevelNames.append(dataDict[seriesNamesFromDataDict[0]][i]['factorLevel'])
    
    #setup plot
    fig = plt.figure(figsize=(figwidth, figheight))
    ax = fig.add_subplot(111)
    
    #calculate series data and confidence interval dictionaries
    #keys are the series names (condition-base condition) and the values is a list of all confidence interval properties (mean, upper, lower)
    #if there is no interaction factor, the lists for each series will only have one value
    xValuesDict = {}
    lowerCIValuesDict = {}
    upperCIValuesDict = {}
    #the factor level names are used for the legend
    factorLevelNames = []
    #iterate over the series names to fill the CI information dictionaries for all different levels of the interaction factor
    for currentSeriesName in seriesNamesFromDataDict:
        currentDataList = dataDict[currentSeriesName]
        #set up the empty lists for this series
        xValuesDict[currentSeriesName] = []
        lowerCIValuesDict[currentSeriesName] = []
        upperCIValuesDict[currentSeriesName] = []
        #iterate over the confidence intervals stored for this series (-> all the different levels of the interaction factor)
        for currentSeriesDataDict in currentDataList:
            xValuesDict[currentSeriesName].append(currentSeriesDataDict['mean'])
            lowerCIValuesDict[currentSeriesName].append(currentSeriesDataDict['mean']-currentSeriesDataDict['lower'])
            upperCIValuesDict[currentSeriesName].append(currentSeriesDataDict['upper']-currentSeriesDataDict['mean']) 
            #if the CI dictionary has a key factor level, its an interaction effect -> store the level name of the factor
            if 'factorLevel' in currentSeriesDataDict:
                factorLevelNames.append(currentSeriesDataDict['factorLevel'])
    
    #iterate over the interaction levels to fill the plot with one factor level at a time (displacement is calculated based on the 'i'-value)
    for i in range(numberOfInteractionLevels):
        #create lists to hold the values for the plot of the current level
        xValues = []
        lowerCIValues = []
        upperCIValues = []
        for currentSeriesName in seriesNamesFromDataDict:
            xValues.append(xValuesDict[currentSeriesName][i])
            lowerCIValues.append(lowerCIValuesDict[currentSeriesName][i])
            upperCIValues.append(upperCIValuesDict[currentSeriesName][i])
            
        confidenceIntervals = [lowerCIValues, upperCIValues]
        #select the marker used for the current plots and set the string up for a black color ('k')
        markerString = plotMarker[i % len(plotMarker)] + 'k'
        #fill the plot with information
        #the y-values are offset depending on 'i'
        ax.errorbar(xValues, np.arange(len(dataDict)) + i*(1/(numberOfInteractionLevels+1)), xerr=confidenceIntervals, fmt=markerString, ms=8, mew=1.5, mec='w')
    
    ax.set_xlabel(xLabel)
    #if there is an interaction effect, add a legend to explain the different series
    if numberOfInteractionLevels > 1:
        legendTitle = dataDict[seriesNamesFromDataDict[0]][0]['factor']
        ax.legend(listOfFactorLevelNames, facecolor='w', loc='center', bbox_to_anchor=(-0.2, 0), title = legendTitle)
    
    #configure appearance of the plot
    axisColor = '#8a8a8a'
    ax.spines['right'].set_visible(False)
    ax.spines['top'].set_visible(False)
    ax.spines['left'].set_visible(False)
    ax.spines['bottom'].set_color(axisColor)
    #set the limits of the plot accounting for the levels
    ax.set_ylim(-0.5,len(dataDict)-0.5+0.1*(numberOfInteractionLevels-1))
    if showLineAtZero and not ratioData:
        ax.axvline(x=0, color=axisColor, linewidth=1)   
    if ratioData:
        ax.axvline(x=1, color=axisColor, linewidth=1)
        ax.set_xlim(left=0)
        showOnlyIntegers = True
        
    if showOnlyIntegers:
        ax.xaxis.set_major_locator(MaxNLocator(integer=True))
    ax.tick_params(axis='y', left=False)
    ax.tick_params(axis='x', colors=axisColor)
    
    #if there is an interaction effect, adjust the position of the y labels to be centered for the corresponding CIs
    if numberOfInteractionLevels > 1:
        ax.set_yticks(np.arange(len(dataDict))+(numberOfInteractionLevels-1)/(2*(numberOfInteractionLevels+1)))
    else:
        #otherwise space them normally 
        ax.set_yticks(np.arange(len(dataDict)))
    ax.set_yticklabels(seriesNamesFromDataDict)
                
    # Set the font name for axis tick labels to be Arial
    for tick in ax.get_xticklabels():
        tick.set_fontname("Arial")
        tick.set_fontsize(14)
        tick.set_color(axisColor)
        #tick.set_fontweight('bold')
    for tick in ax.get_yticklabels():
        tick.set_fontname("Arial")
        tick.set_fontsize(14)
        tick.set_fontweight('bold')

    #check if /Figures folder exists and create it if not
    if not os.path.exists('./Figures'):
        os.makedirs('./Figures')
    
    #save plot and show if requested
    fullFilename = './Figures/' + filename
    plt.savefig(fullFilename, transparent=True, bbox_inches='tight', pad_inches=0.125, format='pdf')
    if alsoShowPlot:
        plt.show()